@@ -2,9 +2,11 @@ from __future__ import annotations
 
 import abc
 import asyncio
+import concurrent.futures
 import contextvars
 import datetime
 import functools
+import logging
 import typing
 from typing import (
     Optional,
@@ -23,7 +25,8 @@ from google.protobuf.message import Message
 from google.protobuf.duration_pb2 import Duration as ProtoDuration
 from google.protobuf.timestamp_pb2 import Timestamp as ProtoTimeStamp
 
-import ydb.aio
+from ...driver import Driver
+from ...aio.driver import Driver as DriverIO
 
 # Workaround for good IDE and universal for runtime
 if typing.TYPE_CHECKING:
@@ -32,6 +35,10 @@ else:
     from ..common.protos import ydb_topic_pb2, ydb_issue_message_pb2
 
 from ... import issues, connection
+from ...settings import BaseRequestSettings
+from ..._constants import DEFAULT_LONG_STREAM_TIMEOUT
+
+logger = logging.getLogger(__name__)
 
 
 class IFromProto(abc.ABC):
@@ -111,24 +118,25 @@ class AsyncQueueToSyncIteratorAsyncIO:
         return item
 
 
-class SyncIteratorToAsyncIterator:
-    def __init__(self, sync_iterator: Iterator):
+class SyncToAsyncIterator:
+    def __init__(self, sync_iterator: Iterator, executor: concurrent.futures.Executor):
         self._sync_iterator = sync_iterator
+        self._executor = executor
 
     def __aiter__(self):
         return self
 
     async def __anext__(self):
         try:
-            res = await to_thread(self._sync_iterator.__next__)
+            res = await to_thread(self._sync_iterator.__next__, executor=self._executor)
             return res
-        except StopAsyncIteration:
-            raise StopIteration()
+        except StopIteration:
+            raise StopAsyncIteration()
 
 
 class IGrpcWrapperAsyncIO(abc.ABC):
     @abc.abstractmethod
-    async def receive(self) -> Any:
+    async def receive(self, timeout: Optional[int] = None) -> Any:
         ...
 
     @abc.abstractmethod
@@ -140,7 +148,7 @@ class IGrpcWrapperAsyncIO(abc.ABC):
         ...
 
 
-SupportedDriverType = Union[ydb.Driver, ydb.aio.Driver]
+SupportedDriverType = Union[Driver, DriverIO]
 
 
 class GrpcWrapperAsyncIO(IGrpcWrapperAsyncIO):
@@ -149,12 +157,24 @@ class GrpcWrapperAsyncIO(IGrpcWrapperAsyncIO):
     convert_server_grpc_to_wrapper: Callable[[Any], Any]
     _connection_state: str
     _stream_call: Optional[Union[grpc.aio.StreamStreamCall, "grpc._channel._MultiThreadedRendezvous"]]
+    _wait_executor: Optional[concurrent.futures.ThreadPoolExecutor]
 
     def __init__(self, convert_server_grpc_to_wrapper):
         self.from_client_grpc = asyncio.Queue()
         self.convert_server_grpc_to_wrapper = convert_server_grpc_to_wrapper
         self._connection_state = "new"
         self._stream_call = None
+        self._wait_executor = None
+
+        self._stream_settings: BaseRequestSettings = (
+            BaseRequestSettings()
+            .with_operation_timeout(DEFAULT_LONG_STREAM_TIMEOUT)
+            .with_cancel_after(DEFAULT_LONG_STREAM_TIMEOUT)
+            .with_timeout(DEFAULT_LONG_STREAM_TIMEOUT)
+        )
+
+    def __del__(self):
+        self._clean_executor(wait=False)
 
     async def start(self, driver: SupportedDriverType, stub, method):
         if asyncio.iscoroutinefunction(driver.__call__):
@@ -168,32 +188,51 @@ class GrpcWrapperAsyncIO(IGrpcWrapperAsyncIO):
         if self._stream_call:
             self._stream_call.cancel()
 
-    async def _start_asyncio_driver(self, driver: ydb.aio.Driver, stub, method):
+        self._clean_executor(wait=True)
+
+    def _clean_executor(self, wait: bool):
+        if self._wait_executor:
+            self._wait_executor.shutdown(wait)
+
+    async def _start_asyncio_driver(self, driver: DriverIO, stub, method):
         requests_iterator = QueueToIteratorAsyncIO(self.from_client_grpc)
         stream_call = await driver(
             requests_iterator,
             stub,
             method,
+            settings=self._stream_settings,
         )
         self._stream_call = stream_call
         self.from_server_grpc = stream_call.__aiter__()
 
-    async def _start_sync_driver(self, driver: ydb.Driver, stub, method):
+    async def _start_sync_driver(self, driver: Driver, stub, method):
         requests_iterator = AsyncQueueToSyncIteratorAsyncIO(self.from_client_grpc)
+        self._wait_executor = concurrent.futures.ThreadPoolExecutor(max_workers=1)
+
         stream_call = await to_thread(
             driver,
             requests_iterator,
             stub,
             method,
+            executor=self._wait_executor,
+            settings=self._stream_settings,
         )
         self._stream_call = stream_call
-        self.from_server_grpc = SyncIteratorToAsyncIterator(stream_call.__iter__())
+        self.from_server_grpc = SyncToAsyncIterator(stream_call.__iter__(), self._wait_executor)
 
-    async def receive(self) -> Any:
+    async def receive(self, timeout: Optional[int] = None) -> Any:
         # todo handle grpc exceptions and convert it to internal exceptions
         try:
-            grpc_message = await self.from_server_grpc.__anext__()
-        except grpc.RpcError as e:
+            if timeout is None:
+                grpc_message = await self.from_server_grpc.__anext__()
+            else:
+
+                async def get_response():
+                    return await self.from_server_grpc.__anext__()
+
+                grpc_message = await asyncio.wait_for(get_response(), timeout)
+
+        except (grpc.RpcError, grpc.aio.AioRpcError) as e:
             raise connection._rpc_error_handler(self._connection_state, e)
 
         issues._process_response(grpc_message)
@@ -255,7 +294,7 @@ def callback_from_asyncio(callback: Union[Callable, Coroutine]) -> [asyncio.Futu
         return loop.run_in_executor(None, callback)
 
 
-async def to_thread(func, /, *args, **kwargs):
+async def to_thread(func, *args, executor: Optional[concurrent.futures.Executor], **kwargs):
     """Asynchronously run function *func* in a separate thread.
 
     Any *args and **kwargs supplied for this function are directly passed
@@ -271,22 +310,25 @@ async def to_thread(func, /, *args, **kwargs):
     loop = asyncio.get_running_loop()
     ctx = contextvars.copy_context()
     func_call = functools.partial(ctx.run, func, *args, **kwargs)
-    return await loop.run_in_executor(None, func_call)
+    return await loop.run_in_executor(executor, func_call)
 
 
-def proto_duration_from_timedelta(t: Optional[datetime.timedelta]) -> ProtoDuration:
+def proto_duration_from_timedelta(t: Optional[datetime.timedelta]) -> Optional[ProtoDuration]:
     if t is None:
         return None
+
     res = ProtoDuration()
     res.FromTimedelta(t)
+    return res
 
 
-def proto_timestamp_from_datetime(t: Optional[datetime.datetime]) -> ProtoTimeStamp:
+def proto_timestamp_from_datetime(t: Optional[datetime.datetime]) -> Optional[ProtoTimeStamp]:
     if t is None:
         return None
 
     res = ProtoTimeStamp()
     res.FromDatetime(t)
+    return res
 
 
 def datetime_from_proto_timestamp(
