@@ -18,6 +18,7 @@ from .. import StatusCode, issues
 from .._grpc.grpcwrapper.ydb_topic import (
     Codec,
     StreamWriteMessage,
+    TransactionIdentity,
     UpdateTokenRequest,
     UpdateTokenResponse,
 )
@@ -42,6 +43,14 @@ from .topic_writer_asyncio import (
 
 from ..credentials import AnonymousCredentials
 
+from .._constants import DEFAULT_INITIAL_RESPONSE_TIMEOUT
+
+
+FAKE_TRANSACTION_IDENTITY = TransactionIdentity(
+    tx_id="transaction_id",
+    session_id="session_id",
+)
+
 
 @pytest.fixture
 def default_driver() -> aio.Driver:
@@ -132,7 +141,7 @@ class TestWriterAsyncIOStream:
 
     async def test_write_a_message(self, writer_and_stream: WriterWithMockedStream):
         data = "123".encode()
-        now = datetime.datetime.now()
+        now = datetime.datetime.now(datetime.timezone.utc)
         writer_and_stream.writer.write(
             [
                 InternalMessage(
@@ -148,11 +157,50 @@ class TestWriterAsyncIOStream:
         expected_message = StreamWriteMessage.FromClient(
             StreamWriteMessage.WriteRequest(
                 codec=Codec.CODEC_RAW,
+                tx_identity=None,
                 messages=[
                     StreamWriteMessage.WriteRequest.MessageData(
                         seq_no=1,
                         created_at=now,
                         data=data,
+                        metadata_items={},
+                        uncompressed_size=len(data),
+                        partitioning=None,
+                    )
+                ],
+            )
+        )
+
+        sent_message = await writer_and_stream.stream.from_client.get()
+        assert expected_message == sent_message
+
+    async def test_write_a_message_with_tx(self, writer_and_stream: WriterWithMockedStream):
+        writer_and_stream.writer._tx_identity = FAKE_TRANSACTION_IDENTITY
+
+        data = "123".encode()
+        now = datetime.datetime.now(datetime.timezone.utc)
+        writer_and_stream.writer.write(
+            [
+                InternalMessage(
+                    PublicMessage(
+                        seqno=1,
+                        created_at=now,
+                        data=data,
+                    )
+                )
+            ]
+        )
+
+        expected_message = StreamWriteMessage.FromClient(
+            StreamWriteMessage.WriteRequest(
+                codec=Codec.CODEC_RAW,
+                tx_identity=FAKE_TRANSACTION_IDENTITY,
+                messages=[
+                    StreamWriteMessage.WriteRequest.MessageData(
+                        seq_no=1,
+                        created_at=now,
+                        data=data,
+                        metadata_items={},
                         uncompressed_size=len(data),
                         partitioning=None,
                     )
@@ -183,6 +231,57 @@ class TestWriterAsyncIOStream:
         receive_task.cancel()
         await asyncio.wait([receive_task])
 
+        await writer.close()
+
+    async def test_init_timeout_parameter(self, stream):
+        """Test that WriterAsyncIOStream._start calls stream.receive with timeout=10"""
+        writer_id = 1
+        settings = WriterSettings(PublicWriterSettings("test-topic", "test-producer"))
+
+        # Mock stream.receive to check if timeout is passed
+        with mock.patch.object(stream, "receive") as mock_receive:
+            mock_receive.return_value = StreamWriteMessage.InitResponse(
+                last_seq_no=0,
+                session_id="test_session",
+                partition_id=1,
+                supported_codecs=[Codec.CODEC_RAW],
+                status=ServerStatus(StatusCode.SUCCESS, []),
+            )
+
+            writer = WriterAsyncIOStream(writer_id, settings)
+            await writer._start(stream, settings.create_init_request())
+
+            # Verify that receive was called with timeout
+            mock_receive.assert_called_with(timeout=DEFAULT_INITIAL_RESPONSE_TIMEOUT)
+
+        await writer.close()
+
+    async def test_init_timeout_behavior(self, stream):
+        """Test that WriterAsyncIOStream._start raises TopicWriterError when receive times out"""
+        writer_id = 1
+        settings = WriterSettings(PublicWriterSettings("test-topic", "test-producer"))
+
+        # Mock stream.receive to directly raise TimeoutError when called with timeout
+        async def timeout_receive(timeout=None):
+            if timeout == DEFAULT_INITIAL_RESPONSE_TIMEOUT:
+                raise asyncio.TimeoutError("Simulated timeout")
+            return StreamWriteMessage.InitResponse(
+                last_seq_no=0,
+                session_id="test_session",
+                partition_id=1,
+                supported_codecs=[Codec.CODEC_RAW],
+                status=ServerStatus(StatusCode.SUCCESS, []),
+            )
+
+        with mock.patch.object(stream, "receive", side_effect=timeout_receive):
+            writer = WriterAsyncIOStream(writer_id, settings)
+
+            # Should raise TopicWriterError with timeout message
+            with pytest.raises(TopicWriterError, match="Timeout waiting for init response"):
+                await writer._start(stream, settings.create_init_request())
+
+        # Don't close writer since _start failed and _stream was never set
+
 
 @pytest.mark.asyncio
 class TestWriterAsyncIOReconnector:
@@ -203,6 +302,7 @@ class TestWriterAsyncIOReconnector:
             update_token_interval: Optional[int, float] = None,
             get_token_function: Optional[Callable[[], str]] = None,
         ):
+            self._id = 0
             self.last_seqno = 0
             self.from_server = asyncio.Queue()
             self.from_client = asyncio.Queue()
@@ -261,7 +361,7 @@ class TestWriterAsyncIOReconnector:
 
         res = DoubleQueueWriters()
 
-        async def async_create(driver, init_message, token_getter):
+        async def async_create(driver, init_message, token_getter, tx_identity):
             return res.get_first()
 
         monkeypatch.setattr(WriterAsyncIOStream, "create", async_create)
@@ -320,7 +420,7 @@ class TestWriterAsyncIOReconnector:
         get_stream_writer,
         default_write_statistic,
     ):
-        now = datetime.datetime.now()
+        now = datetime.datetime.now(datetime.timezone.utc)
         data = "123".encode()
 
         message1 = PublicMessage(
@@ -439,8 +539,11 @@ class TestWriterAsyncIOReconnector:
 
         await reconnector.close(flush=False)
 
-    async def test_deny_double_seqno(self, reconnector: WriterAsyncIOReconnector):
+    async def test_deny_double_seqno(self, reconnector: WriterAsyncIOReconnector, get_stream_writer):
+        writer = get_stream_writer()
+
         await reconnector.write_with_ack_future([PublicMessage(seqno=10, data="123")])
+        writer.from_server.put_nowait(self.make_default_ack_message(seq_no=10))
 
         with pytest.raises(TopicWriterError):
             await reconnector.write_with_ack_future([PublicMessage(seqno=9, data="123")])
@@ -449,12 +552,13 @@ class TestWriterAsyncIOReconnector:
             await reconnector.write_with_ack_future([PublicMessage(seqno=10, data="123")])
 
         await reconnector.write_with_ack_future([PublicMessage(seqno=11, data="123")])
+        writer.from_server.put_nowait(self.make_default_ack_message(seq_no=11))
 
-        await reconnector.close(flush=False)
+        await reconnector.close(flush=True)
 
     @freezegun.freeze_time("2022-01-13 20:50:00", tz_offset=0)
     async def test_auto_created_at(self, default_driver, default_settings, get_stream_writer):
-        now = datetime.datetime.now()
+        now = datetime.datetime.now(datetime.timezone.utc)
 
         settings = copy.deepcopy(default_settings)
         settings.auto_created_at = True
@@ -538,7 +642,7 @@ class TestWriterAsyncIOReconnector:
             mess = mess[0]
 
             assert mess.codec == expected_codecs[i]
-            assert mess.get_bytes() == expected_datas[i]
+            assert mess.get_data_bytes() == expected_datas[i]
 
         await reconnector.close(flush=False)
 
@@ -569,7 +673,9 @@ class TestWriterAsyncIOReconnector:
 
         for index, mess in enumerate(messages):
             assert mess.codec == codec
-            assert mess.get_bytes() == expected_datas[index]
+            assert mess.get_data_bytes() == expected_datas[index]
+
+        await reconnector.close(flush=True)
 
     async def test_custom_encoder(self, default_driver, default_settings, get_stream_writer):
         codec = 10001
@@ -579,7 +685,7 @@ class TestWriterAsyncIOReconnector:
         settings.codec = codec
         reconnector = WriterAsyncIOReconnector(default_driver, settings)
 
-        now = datetime.datetime.now()
+        now = datetime.datetime.now(datetime.timezone.utc)
         seqno = self.init_last_seqno + 1
 
         await reconnector.write_with_ack_future([PublicMessage(data=b"123", seqno=seqno, created_at=now)])
@@ -617,7 +723,7 @@ class TestWriterAsyncIO:
                 self.messages_writted.set()
                 return futures
 
-        async def close(self):
+        async def close(self, flush: bool):
             pass
 
     @pytest.fixture
