@@ -1,5 +1,14 @@
-from ydb._grpc.common.protos import ydb_scripting_pb2
-from ydb._grpc.common import ydb_scripting_v1_pb2_grpc
+import typing
+
+# Workaround for good IDE and universal for runtime
+if typing.TYPE_CHECKING:
+    from ._grpc.v4.protos import ydb_scripting_pb2
+    from ._grpc.v4 import ydb_scripting_v1_pb2_grpc
+else:
+    from ._grpc.common.protos import ydb_scripting_pb2
+    from ._grpc.common import ydb_scripting_v1_pb2_grpc
+
+
 from . import issues, convert, settings
 
 
@@ -39,19 +48,13 @@ class ExplainYqlScriptSettings(settings.BaseRequestSettings):
 
 
 def _execute_yql_query_request_factory(script, tp=None, settings=None):
-    params = (
-        None
-        if tp is None
-        else convert.parameters_to_pb(tp.parameters_types, tp.parameters_values)
-    )
+    params = None if tp is None else convert.parameters_to_pb(tp.parameters_types, tp.parameters_values)
     return ydb_scripting_pb2.ExecuteYqlRequest(script=script, parameters=params)
 
 
 class YqlQueryResult(object):
     def __init__(self, result, scripting_client_settings=None):
-        self.result_sets = convert.ResultSets(
-            result.result_sets, scripting_client_settings
-        )
+        self.result_sets = convert.ResultSets(result.result_sets, scripting_client_settings)
 
 
 class YqlExplainResult(object):
@@ -77,9 +80,7 @@ class ScriptingClient(object):
     def __init__(self, driver, scripting_client_settings=None):
         self.driver = driver
         self.scripting_client_settings = (
-            scripting_client_settings
-            if scripting_client_settings is not None
-            else ScriptingClientSettings()
+            scripting_client_settings if scripting_client_settings is not None else ScriptingClientSettings()
         )
 
     def execute_yql(self, script, typed_parameters=None, settings=None):
