@@ -38,10 +38,7 @@ try:
 
         def _requires_quotes(self, value):
             # Force all identifiers to get quoted unless already quoted.
-            return not (
-                value.startswith(self.initial_quote)
-                and value.endswith(self.final_quote)
-            )
+            return not (value.startswith(self.initial_quote) and value.endswith(self.final_quote))
 
     class YqlTypeCompiler(GenericTypeCompiler):
         def visit_VARCHAR(self, type_, **kw):
@@ -126,9 +123,7 @@ try:
             name = "::".join(name_parts)
             params = func.params_expr._compiler_dispatch(self, **kwargs)
             args = self.function_argspec(func, **kwargs)
-            return "%(name)s%(params)s%(args)s" % dict(
-                name=name, params=params, args=args
-            )
+            return "%(name)s%(params)s%(args)s" % dict(name=name, params=params, args=args)
 
         def visit_function(self, func, add_to_result_map=None, **kwargs):
             # Copypaste of `sa.sql.compiler.SQLCompiler.visit_function` with
@@ -191,11 +186,16 @@ try:
         ydb.PrimitiveType.DyNumber: sa.TEXT,
     }
 
-    def _get_column_type(t):
-        if isinstance(t.item, ydb.DecimalType):
-            return sa.DECIMAL(precision=t.item.precision, scale=t.item.scale)
+    def _get_column_info(t):
+        nullable = False
+        if isinstance(t, ydb.OptionalType):
+            nullable = True
+            t = t.item
 
-        return COLUMN_TYPES[t.item]
+        if isinstance(t, ydb.DecimalType):
+            return sa.DECIMAL(precision=t.precision, scale=t.scale), nullable
+
+        return COLUMN_TYPES[t], nullable
 
     class YqlDialect(DefaultDialect):
         name = "yql"
@@ -250,11 +250,12 @@ try:
             columns = raw_conn.describe(qt)
             as_compatible = []
             for column in columns:
+                col_type, nullable = _get_column_info(column.type)
                 as_compatible.append(
                     {
                         "name": column.name,
-                        "type": _get_column_type(column.type),
-                        "nullable": True,
+                        "type": col_type,
+                        "nullable": nullable,
                     }
                 )
 
